#ifndef _LED_CUBE_AT90USB_H_
#define _LED_CUBE_AT90USB_H_

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/power.h>
#include <avr/interrupt.h>
#include <string.h>
#include <stdio.h>
#include <util/delay.h>

#include <LUFA/Version.h>
#include <LUFA/Drivers/USB/USB.h>
#include <LUFA/Drivers/USB/Class/CDC.h>

#define REQ_GetLineEncoding          0x21
#define REQ_SetLineEncoding          0x20
#define REQ_SetControlLineState      0x22

#define NOTIF_SerialState            0x20

#define CONTROL_LINE_OUT_DTR         (1 << 0)
#define CONTROL_LINE_OUT_RTS         (1 << 1)
#define CONTROL_LINE_IN_DCD          (1 << 0)
#define CONTROL_LINE_IN_DSR          (1 << 1)
#define CONTROL_LINE_IN_BREAK        (1 << 2)
#define CONTROL_LINE_IN_RING         (1 << 3)
#define CONTROL_LINE_IN_FRAMEERROR   (1 << 4)
#define CONTROL_LINE_IN_PARITYERROR  (1 << 5)
#define CONTROL_LINE_IN_OVERRUNERROR (1 << 6)

typedef struct
{
	uint32_t BaudRateBPS;
	uint8_t  CharFormat;                    
	uint8_t  ParityType; 
	uint8_t  DataBits;
} CDC_Line_Coding_t;


typedef struct
{
	uint8_t  NotificationType;
	uint8_t  Notification;
	uint16_t wValue;
	uint16_t wIndex;
	uint16_t wLength;
} USB_Notification_Header_t;


enum CDCDevice_CDC_LineCodingFormats_t
{
	OneStopBit          = 0,
	OneAndAHalfStopBits = 1,
	TwoStopBits         = 2,
};

enum CDCDevice_LineCodingParity_t
{
	Parity_None         = 0,
	Parity_Odd          = 1,
	Parity_Even         = 2,
	Parity_Mark         = 3,
	Parity_Space        = 4,
};


void CDC_USBTask(USB_ClassInfo_CDC_Device_t*);
void EVENT_USB_Device_ConfigurationChanged(void);
void EVENT_USB_Device_UnhandledControlRequest(void);

#endif
